
/*
Split-Split (+Block) Experimental Design from page 191 of Reza Hoshmand (1994).
Experimental Research Design and Analysis. CRC Press.

Corn yield (y, response variable) in relation to:
	hybrid - whole plot,
	row spacing (row), sub-plot,
	plant density (density) - sub-sub-plot, and
	block, a random effects variable.
All factors are fixed effects except for block. Other random effects (involving
interactions with block) are needed to account for the experimental design 
(variation among subjects [experimental units] at the whole-plot, sub-plot, and 
sub-sub-plot levels).

This example is used mostly to demonstrate an exploration of the variance terms, and
what to do when a variance estimate is 0.
*/

data spl_spl;
input y hybrid row density block;
datalines;
140  1  12  1  1
138  1  12  1  2
130  1  12  1  3
142  1  12  1  4
145  1  12  2  1
146  1  12  2  2
150  1  12  2  3
147  1  12  2  4
150  1  12  3  1
149  1  12  3  2
146  1  12  3  3
150  1  12  3  4
136  1  25  1  1
132  1  25  1  2
134  1  25  1  3
138  1  25  1  4
140  1  25  2  1
134  1  25  2  2
136  1  25  2  3
140  1  25  2  4
145  1  25  3  1
138  1  25  3  2
138  1  25  3  3
142  1  25  3  4
142  2  12  1  1
132  2  12  1  2
128  2  12  1  3
140  2  12  1  4
146  2  12  2  1
136  2  12  2  2
140  2  12  2  3
141  2  12  2  4
148  2  12  3  1
140  2  12  3  2
142  2  12  3  3
140  2  12  3  4
132  2  25  1  1
130  2  25  1  2
136  2  25  1  3
134  2  25  1  4
138  2  25  2  1
132  2  25  2  2
130  2  25  2  3
132  2  25  2  4
140  2  25  3  1
134  2  25  3  2
130  2  25  3  3
136  2  25  3  4
;
run;

title 'Mixed-model analysis of split-split plot (with blocks) ';
ods html;
ods graphics on;
proc glimmix data=spl_spl plots=studentpanel; 
  title2 '(random statement with three terms (plus implied residual))';
  class hybrid row density block;
  model y = hybrid|row|density / ddfm=KR;
  random block block*hybrid  row(block hybrid);
  *lsmeans hybrid|row|density ;
run;
ods graphics off;
ods html close;
run;


*^--In above fit, one of the variance estimates is 0. It should be pointed out that a
	positive variance actually means that observations within a subject (e.g., block) 
	are positively correlated. This is the general expectation. A 0 variance means that
	observations within a subject are uncorrelated (independent). It is quite controversial 
	about what should be done when a variance is estimated as 0 .....;
*---One approach is to ignore the issue. For balanced designs and models with just
	variance components (as here), this may be just fine. However, model fits _can_
	be unstable with a 0 variance. Thus, some argue to take out random effects 
	associated with a 0 variance, and re-fit the model. This approach would fall 
	under the heading of a "Model Building" statitical protocol, where one is 
	using mixed-model methodology to find and fit the most approriate mixed model for 
	the data, given the treatment and experimental structure. ;
*---The take-it-out approach generally will not affect model LSMEANS and SEs, but it 
	may have some influence on the denominator df (depending on the ddfm= option).;
ods html;
ods graphics on;
proc glimmix data=spl_spl plots=studentpanel; 
  title '(take out the 0 variance term)';
  class hybrid row density block;
  model y = hybrid|row|density / ddfm=KR;
  random block block*hybrid  /*row(block hybrid)*/;  *<--take out the sub-plot variation term;
  lsmeans row*density / slice=(row density) plot=(mean(sliceby=row cl join ));
run;
ods graphics off;
ods html close;
run;


*---Another approach is to allow for _negative_ variances when the initial fit gives 0 for 
	one or more estimated variances! A negative variance may seem nonsensical, and it
	is true that many statisticians will not take this approach, especially those who think
	in terms of well-defined likelihood functions. However, a negative
	variance means that observations within a subject (experimental unit, such as block) have 
	negative correlation (which is _not_ nonsensical).;
*---Use NOBOUND option to allow for negative variances.;
ods html;
ods graphics on;
proc glimmix data=spl_spl NOBOUND ; 
  title '(allow for negative variances using NOBOUND)';
  class hybrid row density block;
  model y = hybrid|row|density / ddfm=KR;
  random block block*hybrid  row(block hybrid);
  lsmeans row*density / slice=(row density) plot=(mean(sliceby=row cl join ));
run;
ods graphics off;
ods html close;
run;

*---One argument in favor of negative variances is that for certain circumstances
	(e.g., completely balanced, no special correlation structures), the mixed-model-
	based analysis will duplicate the old-fashioned (traditional) ANOVA results, which
	do not involve likelihoods. One then gets the denominator degrees of freedom (df) that
	reflect the experimental design (and is not affected by the 'results'). When one has a 
	0 variance estimate, the denominator df will reflect the fitted mixed model (including 
	just the nonzero random effects), not necessarily the experimental design.;
*---One can get the old-fashioned ANOVA-like results using PROC MIXED, by choosing the
	TYPE3 estimation method. Without this method= option, you would get the restricted
	maximum likelihood (REML) model fit, identical to what you would obtain using
	GLIMMIX (see above).;
ods html;
ods graphics on;
proc mixed data=spl_spl nobound method=type3; 
  title '(a traditional ANOVA like analysis)';
  class hybrid row density block;
  model y = hybrid|row|density;
  random block block*hybrid  row(block hybrid);
  *lsmeans hybrid|row|density ;
run;
ods graphics off;
ods html close;
run;


/*
Personally, I have used all of these approaches, depending on the immediate objectives of 
the investigation. I am more inclined to _not_ allow for negative variances, reflecting my
general interest in describing the data through a likelihood function. However, there are
exceptions. Moreover, in more complex situations, one may not be able to get a stable
fit, or achieve convergence of the iterative optimization procedure, with one of the approaches.
In those cases, you may only be able to obtain a reasonable model fit with just one
of the approaches. 
It is probably a good idea for investigators to report when they use the NOBOUND option.

Note: the denominator df method (ddfm=) also has an affect on these choices.

*/

